@@ -41,7 +41,7 @@
  *              permission of John Bradley.
  */
 
-#include <varargs.h>
+#include <stdarg.h>
 
 #include <stdio.h>
 #include <ctype.h>
@@ -153,86 +153,74 @@ ARG1 (HTMLWidget, hw)
 }
 
 
-/*__________________________________________________________________________
- |
- | PSprintf - dynamic string concatenation function.
- |
- |  In successive calls, the formatted string will be appended to the global
- |  output string Sp.
- |  It assumes that on each call, the length of the text appended to Sp
- |  is less than 1024.
- |  The format string is used as in printf, so you can use additional
- |  arguments.
- |
- |  When successful, PSprintf returns the number of characters printed
- |  in this call, otherwise it returns EOF (just as printf does)
- |
-*/
+/* PSprintf - dynamic string concatenation function.
+ *
+ *  In successive calls, the formatted string will be appended to the global
+ *  output string Sp.
+ *  It assumes that on each call, the length of the text appended to Sp
+ *  is less than 1024.
+ *  The format string is used as in printf, so you can use additional
+ *  arguments.
+ *
+ *  When successful, PSprintf returns the number of characters printed
+ *  in this call, otherwise it returns EOF (just as printf does)
+ */
 
-#ifdef BROKEN_SOLARIS_COMPILER_STDARG
-/* "Looks like there's a bug in Sun's C compiler and the stdarg.h use
-   of va_start() in HTML-PSformat.c. Until the SunPro folks can take a
-   look at the problem, the following pre-ANSI code should workaround
-   the problem." */
-static int PSprintf 
-ARG1V (char *, format,...)
+#ifdef OLDC
+static int PSprintf(format, va_alist)
+    char * format;
+    va_dcl
+{
+    int 	len;
+    char 	*s;
+    va_list	ap;
+
+    if (PS_size - PS_len < 1024) {
+	PS_size += 1024;
+	if ((s = (char *) realloc(PS_string, PS_size)) == NULL) {
+#ifdef VERBOSE
+		fprintf(stderr, "PSprintf malloc failed\n");
+#endif
+		return(EOF);
+	}
+	PS_string = s;
+    }
+    va_start(ap);
+    len = (int) vsprintf(PS_string+PS_len, format, ap);
+    /* this is a hack to make it work on systems were vsprintf(s,...)
+     * returns s, instead of the len.  */
+    if (len != EOF && len != (int) NULL) 
+	PS_len += strlen(PS_string+PS_len);
+    va_end(ap);
+    return(len);
+}
+#else
+static int PSprintf(char *format, ...)       
 {
-  va_dcl
     va_list args;
-  int len;
-  char *s;
-
-  if (PS_size - PS_len < 1024)
-  {
-    PS_size += 1024;
-    if ((s = (char *) XtRealloc (PS_string, PS_size)) == NULL)
-    {
-      fprintf (stderr, "PSprintf malloc failed\n");
-      return (EOF);
-    }
-    PS_string = s;
-  }
-  va_start (args);
-  len = vsprintf (PS_string + PS_len, format, args);
-  /* this is a hack to make it work on systems were vsprintf(s,.)
-   * returns s, instead of the len.
-   */
-  if (len != EOF && len != NULL)
-    PS_len += strlen (PS_string + PS_len);
-  va_end (args);
-  return (len);
+    int     len;        
+    char    *s;                 
+    va_start(args, format);
+  
+    if (PS_size - PS_len < 1024) {
+        PS_size += 1024;
+        if ((s = (char *) realloc(PS_string, PS_size)) == NULL) {
+#ifdef VERBOSE
+		fprintf(stderr, "PSprintf malloc failed\n");
+#endif
+                return(EOF);
+        }
+        PS_string = s;
+    }           
+    len = (int) vsprintf(PS_string+PS_len, format, args);
+    /* this is a hack to make it work on systems were vsprintf(s,.)
+     * returns s, instead of the len. */
+    if (len != EOF && len != (int) NULL)
+        PS_len += strlen(PS_string+PS_len);
+    va_end(args);
+    return(len);
 }
-#else /* not BROKEN_SOLARIS_COMPILER_STDARG */
-static int
-PSprintf (format, va_alist)
-char *format;
-va_dcl
-{
-  int len;
-  char *s;
-  va_list args;
-
-  if (PS_size - PS_len < 1024)
-  {
-    PS_size += 1024;
-    if ((s = (char *) XtRealloc (PS_string, PS_size)) == NULL)
-    {
-      fprintf (stderr, "PSprintf malloc failed\n");
-      return (EOF);
-    }
-    PS_string = s;
-  }
-  va_start (args);
-  len = vsprintf (PS_string + PS_len, format, args);
-  /* this is a hack to make it work on systems were vsprintf(s,...)
-   * returns s, instead of the len.
-   */
-  if (len != EOF && len != 0)
-    PS_len += strlen (PS_string + PS_len);
-  va_end (args);
-  return (len);
-}
-#endif /* not BROKEN_SOLARIS_COMPILER_STDARG */
+#endif
 
 /*__________________________________________________________________________
  |
@@ -736,7 +724,9 @@ ARG2 (String, t, int, underline)
 
   if (t2 == NULL)
   {
+#ifdef VERBOSE
     fprintf (stderr, "PStext malloc failed\n");
+#endif
     return;
   }
 
@@ -1392,7 +1382,9 @@ ARG2 (ImageInfo *, img, int, anchor)
     rleline = (unsigned char *) XtMalloc (w * 2);
     if (!rleline)
     {
+#ifdef VERBOSE
       fprintf (stderr, "failed to malloc space for rleline\n");
+#endif
       return;
     }
 
@@ -1441,10 +1433,9 @@ ARG2 (ImageInfo *, img, int, anchor)
 */
 
 String 
-ParseTextToPSString (hw, elist, startp, endp, start_pos, end_pos,
+ParseTextToPSString (hw, startp, endp, start_pos, end_pos,
 		     space_width, lmargin, fontfamily)
 HTMLWidget hw;
-struct ele_rec *elist;
 struct ele_rec *startp;
 struct ele_rec *endp;
 int start_pos, end_pos;
@@ -1645,7 +1636,7 @@ int fontfamily;
       {
 
 	PSmove_offset (eptr->y_offset);
-	PSimage (eptr->pic_data, (eptr->anchorHRef != NULL));
+	PSimage (eptr->img_data->pic_data, (eptr->anchorHRef != NULL));
 	break;
       }
 
