@@ -1,3 +1,5 @@
+/* substantially modified by WBE, Spring 1997, and by others before me */
+
 /****************************************************************************
  * NCSA Mosaic for the X Window System                                      *
  * Software Development Group                                               *
@@ -102,49 +104,6 @@ Colormap colormap;
 }
 
 
-/*
- * Free up all the pixmaps allocated for this document.
- */
-void
-FreeImages (hw)
-HTMLWidget hw;
-{
-  struct ele_rec *eptr;
-
-  eptr = hw->html.formatted_elements;
-  while (eptr != NULL)
-  {
-    if ((eptr->type == E_IMAGE) && (eptr->pic_data != NULL))
-    {
-      /*
-       * Don't free the no_image default image
-       */
-      if ((eptr->pic_data->image != (Pixmap) NULL) &&
-	  (eptr->pic_data->image != delayed_image.image) &&
-	  (eptr->pic_data->image != anchored_image.image) &&
-	  (eptr->pic_data->image != no_image.image))
-      {
-	/*
-	 * Don't free internal images
-	 */
-	if ((eptr->edata != NULL) &&
-	    (strncmp (eptr->edata, INTERNAL_IMAGE,
-		      strlen (INTERNAL_IMAGE)) == 0))
-	{
-	}
-	else
-	{
-	  XFreePixmap (XtDisplay (hw),
-		       eptr->pic_data->image);
-	  eptr->pic_data->image = (Pixmap) NULL;
-	}
-      }
-    }
-    eptr = eptr->next;
-  }
-}
-
-
 /*
  * Find the closest color by allocating it, or picking an already allocated
  * color
@@ -166,10 +125,18 @@ XColor *colr;
   static int have_colors = 0;
   int NumCells;
 
+  NumCells = XDisplayCells(dsp, DefaultScreen(dsp));
+  if (NumCells <= 2)
+  {
+    colr->pixel = (colr->red + colr->green + colr->blue > 98304) ?
+      WhitePixel(dsp, DefaultScreen(dsp)) : BlackPixel(dsp, DefaultScreen(dsp)) ;
+    XQueryColor(dsp, colormap, colr);
+    return;
+  }
+
   match = XAllocColor (dsp, colormap, colr);
   if (match == 0)
   {
-    NumCells = DisplayCells (dsp, DefaultScreen (dsp));
     if (!have_colors)
     {
       for (i = 0; i < NumCells; i++)
@@ -285,8 +252,9 @@ unsigned long ul;
   return -1;
 }
 
+
 /*
- * Make am image of appropriate depth for display from image data.
+ * Make an image of appropriate depth for display from image data.
  */
 XImage *
 MakeImage (dsp, data, width, height, depth, img_info)
@@ -294,10 +262,11 @@ Display *dsp;
 unsigned char *data;
 int width, height;
 int depth;
-ImageInfo *img_info;
+const ImageInfo *img_info;
 {
   int linepad, shiftnum;
   int shiftstart, shiftstop, shiftinc;
+  int bitsperpixel, scanline_pad;
   int bytesperline;
   int temp;
   int w, h;
@@ -308,7 +277,10 @@ ImageInfo *img_info;
   unsigned long c;
   int rshift, gshift, bshift;
 
-  switch (depth)
+  bitsperpixel = _XGetBitsPerPixel(dsp, depth);
+  scanline_pad = _XGetScanlinePad(dsp, depth);
+
+  switch (bitsperpixel)
   {
   case 6:
   case 8:
@@ -327,15 +299,15 @@ ImageInfo *img_info;
     {
       shiftstart = 0;
       shiftstop = 8;
-      shiftinc = depth;
+      shiftinc = bitsperpixel;
     }
     else
     {
-      shiftstart = 8 - depth;
-      shiftstop = -depth;
-      shiftinc = -depth;
+      shiftstart = 8 - bitsperpixel;
+      shiftstop = -bitsperpixel;
+      shiftinc = -bitsperpixel;
     }
-    linepad = 8 - (width % 8);
+    linepad = scanline_pad - (width % scanline_pad);
     bit_data = (unsigned char *) XtMalloc (((width + linepad) * height)
 					   + 1);
     bitp = bit_data;
@@ -367,11 +339,12 @@ ImageInfo *img_info;
 	}
       }
     }
-    bytesperline = (width + linepad) * depth / 8;
+    bytesperline = (width + linepad) * bitsperpixel / 8;
     newimage = XCreateImage (dsp,
 			     DefaultVisual (dsp, DefaultScreen (dsp)),
 			     depth, ZPixmap, 0, (char *) bit_data,
-			     (width + linepad), height, 8, bytesperline);
+			     (width + linepad), height, scanline_pad,
+			     bytesperline);
     break;
     /*
      * WARNING:  This depth 16 code is donated code for 16 bit
@@ -431,6 +404,7 @@ ImageInfo *img_info;
     }
     break;
   case 24:
+  case 32:
     bit_data = (unsigned char *) XtMalloc (width * height * 4);
 
     theVisual = DefaultVisual (dsp, DefaultScreen (dsp));
@@ -488,11 +462,13 @@ HTMLWidget hw;
 }
 
 
+static const char *const ismapformstring = "ISMAP Form";
+
 char *
 IsMapForm (hw)
 HTMLWidget hw;
 {
-  return (XtNewString ("ISMAP Form"));
+  return (XtNewString (ismapformstring));
 }
 
 
@@ -501,22 +477,33 @@ IsIsMapForm (hw, href)
 HTMLWidget hw;
 char *href;
 {
-  if ((href != NULL) && (strcmp (href, "ISMAP Form") == 0))
-  {
-    return (1);
-  }
-  else
-  {
-    return (0);
-  }
+  return (href != NULL  &&
+	  strcmp (href, ismapformstring) == 0);
 }
 
 
+/*
+ * Delayed images are anchored if they aren't already.
+ * This accomplishes several things:
+ * 1) _HTMLInput (which handles mouse button up events) is only called by
+ *     ExtendEnd for button up events in an anchor, and _HTMLInput only
+ *     processes elements that look like anchors;
+ * 2) a standard anchor-style bounding box will be drawn around the delayed
+ *     image icon, so that it's borders are obvious; and
+ * 3) the Anchor Display feature will consider the presence of the mouse
+ *     cursor in the delayed image area to be worth highlighting.
+ *
+ * Clicking on a delayed image will cause the image to be loaded (see
+ * _HTMLInput() ).
+ */
+
+static const char *const delayedhrefstring = "Delayed Image";
+
 char *
 DelayedHRef (hw)
 HTMLWidget hw;
 {
-  return (XtNewString ("Delayed Image"));
+  return (XtNewString (delayedhrefstring));
 }
 
 
@@ -525,51 +512,135 @@ IsDelayedHRef (hw, href)
 HTMLWidget hw;
 char *href;
 {
-  if ((href != NULL) && (strcmp (href, "Delayed Image") == 0))
-  {
-    return (1);
-  }
-  else
-  {
-    return (0);
-  }
+  return (href != NULL  &&
+	  strcmp (href, delayedhrefstring) == 0);
 }
 
 
-Pixmap
-DelayedImage (hw, anchored)
-HTMLWidget hw;
+ImageInfo *
+DelayedImageData (anchored)
 Boolean anchored;
 {
-  if (delayed_image.image == (Pixmap) NULL)
+  static Boolean di_inited = False;
+
+  if (di_inited == False)
   {
-    delayed_image.image = XCreatePixmapFromBitmapData (
-						  XtDisplay (hw->html.view),
-				XtWindow (hw->html.view), DelayedImage_bits,
-				    DelayedImage_width, DelayedImage_height,
-							hw->html.foreground,
-						  hw->core.background_pixel,
-				      DefaultDepthOfScreen (XtScreen (hw)));
+    anchored_image.delayed = True;
+    anchored_image.internal = True;
+    anchored_image.fetched = 0;
+    anchored_image.width = DelayedImage_width;
+    anchored_image.height =
+	DelayedImage_height + AnchoredImage_height + IMAGE_BORDER;
+    anchored_image.num_colors = 0;
+    anchored_image.reds = NULL;
+    anchored_image.greens = NULL;
+    anchored_image.blues = NULL;
+    anchored_image.image_data = NULL;
+    anchored_image.image = (Pixmap) NULL;
+    anchored_image.name = NULL;
+    anchored_image.next = NULL;
+
+    delayed_image.delayed = True;
+    delayed_image.internal = True;
+    delayed_image.fetched = 0;
+    delayed_image.width = DelayedImage_width;
+    delayed_image.height = DelayedImage_height;
+    delayed_image.num_colors = 0;
+    delayed_image.reds = NULL;
+    delayed_image.greens = NULL;
+    delayed_image.blues = NULL;
+    delayed_image.image_data = NULL;
+    delayed_image.image = (Pixmap) NULL;
+    delayed_image.name = NULL;
+    delayed_image.next = NULL;
+
+    di_inited = True;
   }
 
-  if ((anchored == True) && (anchored_image.image == (Pixmap) NULL))
+  return (anchored == True) ?  &anchored_image : &delayed_image;
+}
+
+
+ImageInfo *
+NoImageData ()
+{
+  static Boolean ni_inited = False;
+
+  if (ni_inited == False)
+  {
+    no_image.delayed = 0;
+    no_image.internal = True;
+    no_image.fetched = 0;
+    no_image.width = NoImage_width;
+    no_image.height = NoImage_height;
+    no_image.num_colors = 0;
+    no_image.reds = NULL;
+    no_image.greens = NULL;
+    no_image.blues = NULL;
+    no_image.image_data = NULL;
+    no_image.image = (Pixmap) NULL;
+    no_image.name = NULL;
+    no_image.next = NULL;
+
+    ni_inited = True;
+  }
+
+  return (&no_image);
+}
+
+
+/*
+ * Build pixmaps for the internal images
+ */
+void
+BuildInternalImagePixmaps (hw)
+HTMLWidget hw;
+{
+  if (no_image.image == (Pixmap) NULL)
+  {
+    (void) NoImageData ();	/* make sure noimage has been init'd */
+    no_image.image =
+	XCreatePixmapFromBitmapData (XtDisplay (hw->html.view),
+				     XtWindow (hw->html.view),
+				     NoImage_bits,
+				     NoImage_width, NoImage_height,
+				     hw->html.foreground,
+				     hw->core.background_pixel,
+				     DefaultDepthOfScreen (XtScreen (hw)));
+  }
+
+  if (delayed_image.image == (Pixmap) NULL)
+  {
+    (void) DelayedImageData (False);  /* make sure delayed_image init'd */
+    delayed_image.image =
+	XCreatePixmapFromBitmapData (XtDisplay (hw->html.view),
+				     XtWindow (hw->html.view),
+				     DelayedImage_bits,
+				     DelayedImage_width, DelayedImage_height,
+				     hw->html.foreground,
+				     hw->core.background_pixel,
+				     DefaultDepthOfScreen (XtScreen (hw)));
+  }
+
+  if (anchored_image.image == (Pixmap) NULL)
   {
     Pixmap pix;
 
-    anchored_image.image = XCreatePixmapFromBitmapData (
-						  XtDisplay (hw->html.view),
-			       XtWindow (hw->html.view), AnchoredImage_bits,
-				  AnchoredImage_width, AnchoredImage_height,
-							 hw->html.foreground,
-						  hw->core.background_pixel,
-				      DefaultDepthOfScreen (XtScreen (hw)));
-    pix = XCreatePixmap (
-			  XtDisplay (hw->html.view),
-			  XtWindow (hw->html.view),
-			  DelayedImage_width,
-			  (DelayedImage_height + AnchoredImage_height +
-			   IMAGE_BORDER),
-			  DefaultDepthOfScreen (XtScreen (hw)));
+    (void) DelayedImageData (True);  /* make sure anchored_image init'd */
+    anchored_image.image =
+	XCreatePixmapFromBitmapData (XtDisplay (hw->html.view),
+				     XtWindow (hw->html.view),
+				     AnchoredImage_bits,
+				     AnchoredImage_width, AnchoredImage_height,
+				     hw->html.foreground,
+				     hw->core.background_pixel,
+				     DefaultDepthOfScreen (XtScreen (hw)));
+    pix = XCreatePixmap (XtDisplay (hw->html.view),
+			 XtWindow (hw->html.view),
+			 DelayedImage_width,
+			 (DelayedImage_height + AnchoredImage_height +
+			    IMAGE_BORDER),
+			 DefaultDepthOfScreen (XtScreen (hw)));
     XSetForeground (XtDisplay (hw), hw->html.drawGC,
 		    hw->core.background_pixel);
     XFillRectangle (XtDisplay (hw->html.view), pix,
@@ -586,96 +657,16 @@ Boolean anchored;
 	       0, 0, DelayedImage_width, DelayedImage_height,
 	       0, (AnchoredImage_height + IMAGE_BORDER));
     XFreePixmap (XtDisplay (hw->html.view), anchored_image.image);
+
     anchored_image.image = pix;
-
-    return (anchored_image.image);
   }
-
-  return (delayed_image.image);
-}
-
-
-ImageInfo *
-DelayedImageData (hw, anchored)
-HTMLWidget hw;
-Boolean anchored;
-{
-  delayed_image.delayed = 1;
-  delayed_image.internal = 0;
-  delayed_image.fetched = 0;
-  delayed_image.width = DelayedImage_width;
-  delayed_image.height = DelayedImage_height;
-  delayed_image.num_colors = 0;
-  delayed_image.reds = NULL;
-  delayed_image.greens = NULL;
-  delayed_image.blues = NULL;
-  delayed_image.image_data = NULL;
-  delayed_image.image = (Pixmap) NULL;
-
-  if (anchored == True)
-  {
-    anchored_image.delayed = 0;
-    anchored_image.internal = 0;
-    anchored_image.fetched = 0;
-    anchored_image.width = DelayedImage_width;
-    anchored_image.height = DelayedImage_height +
-      AnchoredImage_height + IMAGE_BORDER;
-    anchored_image.num_colors = 0;
-    anchored_image.reds = NULL;
-    anchored_image.greens = NULL;
-    anchored_image.blues = NULL;
-    anchored_image.image_data = NULL;
-    anchored_image.image = (Pixmap) NULL;
-
-    return (&anchored_image);
-  }
-
-  return (&delayed_image);
-}
-
-
-Pixmap
-NoImage (hw)
-HTMLWidget hw;
-{
-  if (no_image.image == (Pixmap) NULL)
-  {
-    no_image.image = XCreatePixmapFromBitmapData (
-						   XtDisplay (hw->html.view),
-				     XtWindow (hw->html.view), NoImage_bits,
-					      NoImage_width, NoImage_height,
-						   hw->html.foreground,
-						   hw->core.background_pixel,
-				      DefaultDepthOfScreen (XtScreen (hw)));
-  }
-  return (no_image.image);
-}
-
-
-ImageInfo *
-NoImageData (hw)
-HTMLWidget hw;
-{
-  no_image.delayed = 0;
-  no_image.internal = 0;
-  no_image.fetched = 0;
-  no_image.width = NoImage_width;
-  no_image.height = NoImage_height;
-  no_image.num_colors = 0;
-  no_image.reds = NULL;
-  no_image.greens = NULL;
-  no_image.blues = NULL;
-  no_image.image_data = NULL;
-  no_image.image = (Pixmap) NULL;
-
-  return (&no_image);
 }
 
 
 Pixmap
 InfoToImage (hw, img_info)
 HTMLWidget hw;
-ImageInfo *img_info;
+const ImageInfo *img_info;
 {
   int i, size;
   int delta, not_right_col, not_last_row;
@@ -877,10 +868,8 @@ ImageInfo *img_info;
     if (Img != (Pixmap) NULL)
     {
       XFreePixmap (XtDisplay (hw), Img);
+      Img = (Pixmap) NULL;	/* becomes the result returned */
     }
-    img_info->width = NoImage_width;
-    img_info->height = NoImage_height;
-    Img = NoImage (hw);
   }
   else
   {
@@ -894,3 +883,186 @@ ImageInfo *img_info;
 
   return (Img);
 }
+
+
+/*
+ * Free the ImageInfo block and the all the resources it owns.
+ * Returns the value of pic_data->next as a convenience to the caller.
+ */
+ImageInfo *
+FreeOneImage (hw, pic_data)
+HTMLWidget hw;
+ImageInfo *pic_data;		/* NULL allowed */
+{
+  ImageInfo *next;
+
+  /*
+   * Don't actually free special built-in images
+   * pic_data == NULL is also accepted.
+   */
+  if (pic_data != NULL  &&
+      pic_data->internal == False  &&	 /* the test that ought to work */
+      pic_data != &delayed_image  &&	/* paranoia */
+      pic_data != &anchored_image &&	/* paranoia */
+      pic_data != &no_image)		/* paranoia */
+  {
+    if (pic_data->image_data != NULL) free_mem ((char *)pic_data->image_data);
+    if (pic_data->reds != NULL)       free_mem ((char *)pic_data->reds);
+    if (pic_data->greens != NULL)     free_mem ((char *)pic_data->greens);
+    if (pic_data->blues != NULL)      free_mem ((char *)pic_data->blues);
+    if (pic_data->name != NULL)       free_mem ((char *)pic_data->name);
+    if (pic_data->image != (Pixmap) NULL)
+	XFreePixmap (XtDisplay (hw), pic_data->image);
+
+    next = pic_data->next;
+    free_mem ((char *)pic_data);
+  }
+  else next = NULL;
+
+  return next;
+}
+
+
+/*
+ * Free up all the image data used by the current document.
+ */
+void
+FreeImages (hw)
+HTMLWidget hw;
+{
+  ImageInfo *pic_data = hw->html.image_list;
+
+  while (pic_data != NULL)   pic_data = FreeOneImage (hw, pic_data);
+
+  hw->html.image_list = NULL;
+}
+
+
+/*
+ * Search the list of images for one with NAME.
+ * Return a pointer to an ImageInfo *, or NULL if not found.
+ */
+ImageInfo *
+IsCurrentImage (list, name)
+ImageInfo *list;
+const char *const name;
+{
+  if (list == NULL)
+  {
+    return (NULL);
+  }
+
+  for ( ;  list != NULL;  list = list->next)
+  {
+    if (strcmp (list->name, name) == 0)
+    {
+      return list;
+    }
+  }
+
+  return NULL;
+}
+
+
+/*
+ * Do appropriate things to get the specified image or a suitable substitute.
+ * If a new real image is obtained, it is added to the image_list.
+ * eptr->img_data is created and initialized, and img_data->pic_data is set
+ * to whatever image we're going to display.
+ * This code does not generate pic_data->image; ImageRefresh() does that.
+ */
+void
+HTMLGetImage (hw, eptr, now)
+HTMLWidget hw;
+struct ele_rec *eptr;
+Boolean now;			/* True: force loading now */
+{
+  const char *const edata = eptr->edata;
+  ImageInfo *pic_data;
+
+  /*
+   * Is this image on image_list?
+   */
+  pic_data = IsCurrentImage (hw->html.image_list, edata);
+
+  if (pic_data == NULL)		/* not on image_list; a new image */
+  {
+    if (hw->html.resolveImage != NULL)
+    {
+      /*
+       * arg 3:  True = don't load if downloading is required
+       */
+      pic_data = (* (resolveImageProc) (hw->html.resolveImage) )
+	         (hw, edata, (hw->html.delay_images  &&  !now));
+
+      if (pic_data != NULL)	/* got a real image */
+      {
+	if (pic_data->image_data != NULL)
+	{
+	  pic_data->delayed = 0;
+	  pic_data->name    = XtNewString (edata);
+	  pic_data->fetched = True;
+
+	  /*
+	   * Add pic_data to head of image_list.  Adding to the head helps
+	   * in the common case where ImagePlace() asks for the image twice
+	   * in a row because it didn't fit in the first place tried.
+	   */
+	  pic_data->next = hw->html.image_list;
+	  hw->html.image_list = pic_data;
+	}
+	else			/* well, *that* was useless */
+	{
+	  (void) FreeOneImage (pic_data);
+	  pic_data = NULL;
+	}
+      }
+      /*
+       * The inclusion of && !now is optional.  It's arguable whether to
+       * give the user another try when delayed image loading fails or
+       * leave the user thinking Chimera is broken because it didn't load
+       * the image and redisplays the delayed image icon.  I opted for
+       * making it clear the load failed.  The user still has other ways
+       * to prod Chimera to try again to load the image.
+       */
+      if (pic_data == NULL  &&	/* don't have an image */
+	  hw->html.delay_images  &&  !now)
+      {
+	/*
+	 * Use "delayed image" or "delayed image that's an anchor" box.
+	 */
+	if (eptr->anchorHRef == NULL)  /* image isn't part of an anchor */
+	{
+	  pic_data = DelayedImageData (False);
+	  /*
+	   * Make delayed images look like anchors.
+	   * Why?  See DelayedHRef()  (above).
+	   */
+	  eptr->anchorHRef = DelayedHRef (hw);  /* pseudo-anchor */
+	  eptr->fg = hw->html.anchor_fg;  /* just like a real anchor */
+	}
+	else			/* delayed image that's an anchor */
+	{
+	  pic_data = DelayedImageData (True);
+	}
+      }
+    }
+
+    if (pic_data == NULL)	/* still nothing?  Give up */
+    {
+      pic_data = NoImageData ();  /* never returns NULL */
+    }
+  }
+
+  /*
+   * Create the img_data structure to hold any HTML tag qualifiers.
+   */
+  FreeIMGInfo (eptr->img_data);	 /* free any old img_data; NULL okay */
+
+  eptr->img_data = (IMGInfo *) XtMalloc (sizeof (IMGInfo));
+  eptr->img_data->pic_data = pic_data;
+  
+  /*
+   * Note: never exits with eptr->img_data->pic_data non-NULL
+   */
+}
