@@ -65,7 +65,7 @@
 
 /*
  * Free up the passed linked list of parsed elements, freeing
- * all memory associates with each element.
+ * all memory associated with each element.
  */
 void
 FreeObjList (List)
@@ -181,138 +181,54 @@ int type;
 {
   switch (type)
   {
-  case M_NONE:
-    printf ("M_NONE");
-    break;
-  case M_TITLE:
-    printf ("M_TITLE");
-    break;
-  case M_FIXED:
-    printf ("M_FIXED");
-    break;
-  case M_BOLD:
-    printf ("M_BOLD");
-    break;
-  case M_ITALIC:
-    printf ("M_ITALIC");
-    break;
-  case M_EMPHASIZED:
-    printf ("M_EMPHASIZED");
-    break;
-  case M_STRONG:
-    printf ("M_STRONG");
-    break;
-  case M_CODE:
-    printf ("M_CODE");
-    break;
-  case M_SAMPLE:
-    printf ("M_SAMPLE");
-    break;
-  case M_KEYBOARD:
-    printf ("M_KEYBOARD");
-    break;
-  case M_VARIABLE:
-    printf ("M_VARIABLE");
-    break;
-  case M_CITATION:
-    printf ("M_CITATION");
-    break;
-  case M_STRIKEOUT:
-    printf ("M_STRIKEOUT");
-    break;
-  case M_HEADER_1:
-    printf ("M_HEADER_1");
-    break;
-  case M_HEADER_2:
-    printf ("M_HEADER_2");
-    break;
-  case M_HEADER_3:
-    printf ("M_HEADER_3");
-    break;
-  case M_HEADER_4:
-    printf ("M_HEADER_4");
-    break;
-  case M_HEADER_5:
-    printf ("M_HEADER_5");
-    break;
-  case M_HEADER_6:
-    printf ("M_HEADER_6");
-    break;
-  case M_ANCHOR:
-    printf ("M_ANCHOR");
-    break;
-  case M_PARAGRAPH:
-    printf ("M_PARAGRAPH");
-    break;
-  case M_ADDRESS:
-    printf ("M_ADDRESS");
-    break;
-  case M_PLAIN_TEXT:
-    printf ("M_PLAIN_TEXT");
-    break;
-  case M_LISTING_TEXT:
-    printf ("M_LISTING_TEXT");
-    break;
-  case M_UNUM_LIST:
-    printf ("M_UNUM_LIST");
-    break;
-  case M_NUM_LIST:
-    printf ("M_NUM_LIST");
-    break;
-  case M_MENU:
-    printf ("M_MENU");
-    break;
-  case M_DIRECTORY:
-    printf ("M_DIRECTORY");
-    break;
-  case M_LIST_ITEM:
-    printf ("M_LIST_ITEM");
-    break;
-  case M_DESC_LIST:
-    printf ("M_DESC_LIST");
-    break;
-  case M_DESC_TITLE:
-    printf ("M_DESC_TITLE");
-    break;
-  case M_DESC_TEXT:
-    printf ("M_DESC_TEXT");
-    break;
-  case M_IMAGE:
-    printf ("M_IMAGE");
-    break;
-  case M_SELECT:
-    printf ("M_SELECT");
-    break;
-  case M_OPTION:
-    printf ("M_OPTION");
-    break;
-  case M_INPUT:
-    printf ("M_INPUT");
-    break;
-  case M_TEXTAREA:
-    printf ("M_TEXTAREA");
-    break;
-  case M_FORM:
-    printf ("M_FORM");
-    break;
-  case M_INDEX:
-    printf ("M_INDEX");
-    break;
-  case M_HRULE:
-    printf ("M_HRULE");
-    break;
-  case M_BASE:
-    printf ("M_BASE");
-    break;
-  case M_LINEBREAK:
-    printf ("M_LINEBREAK");
-    break;
-  case M_BLOCKQUOTE:
-    printf ("M_BLOCKQUOTE");
-    break;
-  default:
-    printf ("UNKNOWN %d", type);
-    break;
+  case M_NONE:		printf ("M_NONE");	break;
+  case M_TITLE:		printf ("M_TITLE");	break;
+  case M_FIXED:		printf ("M_FIXED");	break;
+  case M_BOLD:		printf ("M_BOLD");	break;
+  case M_ITALIC:	printf ("M_ITALIC");	break;
+  case M_EMPHASIZED:	printf ("M_EMPHASIZED"); break;
+  case M_STRONG:	printf ("M_STRONG");	break;
+  case M_CODE:		printf ("M_CODE");	break;
+  case M_SAMPLE:	printf ("M_SAMPLE");	break;
+  case M_KEYBOARD:	printf ("M_KEYBOARD");	break;
+  case M_VARIABLE:	printf ("M_VARIABLE");	break;
+  case M_CITATION:	printf ("M_CITATION");	break;
+  case M_STRIKEOUT:	printf ("M_STRIKEOUT"); break;
+  case M_HEADER_1:	printf ("M_HEADER_1");	break;
+  case M_HEADER_2:	printf ("M_HEADER_2");  break;
+  case M_HEADER_3:	printf ("M_HEADER_3");  break;
+  case M_HEADER_4:	printf ("M_HEADER_4");  break;
+  case M_HEADER_5:	printf ("M_HEADER_5");	break;
+  case M_HEADER_6:	printf ("M_HEADER_6");  break;
+  case M_ANCHOR:	printf ("M_ANCHOR");	break;
+  case M_PARAGRAPH:	printf ("M_PARAGRAPH");	break;
+  case M_ADDRESS:	printf ("M_ADDRESS");	break;
+  case M_PLAIN_TEXT:	printf ("M_PLAIN_TEXT"); break;
+  case M_LISTING_TEXT:	printf ("M_LISTING_TEXT"); break;
+  case M_UNUM_LIST:	printf ("M_UNUM_LIST"); break;
+  case M_NUM_LIST:	printf ("M_NUM_LIST");	break;
+  case M_MENU:		printf ("M_MENU");	break;
+  case M_DIRECTORY:	printf ("M_DIRECTORY"); break;
+  case M_LIST_ITEM:	printf ("M_LIST_ITEM"); break;
+  case M_DESC_LIST:	printf ("M_DESC_LIST"); break;
+  case M_DESC_TITLE:	printf ("M_DESC_TITLE"); break;
+  case M_DESC_TEXT:	printf ("M_DESC_TEXT"); break;
+  case M_IMAGE:		printf ("M_IMAGE");	break;
+  case M_SELECT:	printf ("M_SELECT");	break;
+  case M_OPTION:	printf ("M_OPTION");	break;
+  case M_INPUT:		printf ("M_INPUT");	break;
+  case M_TEXTAREA:	printf ("M_TEXTAREA");	break;
+  case M_FORM:		printf ("M_FORM");	break;
+  case M_INDEX:		printf ("M_INDEX");	break;
+  case M_HRULE:		printf ("M_HRULE");	break;
+  case M_BASE:		printf ("M_BASE");	break;
+  case M_LINEBREAK:	printf ("M_LINEBREAK");	break;
+  case M_BLOCKQUOTE:	printf ("M_BLOCKQUOTE"); break;
+  case M_TABLE:		printf ("M_TABLE");	break;
+  case M_TABLE_ROW:	printf ("M_TABLE_ROW"); break;
+  case M_TABLE_ITEM:	printf ("M_TABLE_ITEM"); break;
+  case M_CAPTION:	printf ("M_CAPTION");	break;
+  default:	printf ("UNKNOWN %d", type);	break;
   }
 }
 
@@ -415,32 +331,24 @@ int *cnt;
 /*
  * Free up the passed linked list of formatted elements, freeing
  * all memory associates with each element.
+ * (FreeLineList and MakeLineList are completely unrelated functionally.)
  */
 void
 FreeLineList (list)
 struct ele_rec *list;
 {
-  struct ele_rec *current;
-  struct ele_rec *eptr;
+  struct ele_rec *current = list;
 
-  current = list;
   while (current != NULL)
   {
-    eptr = current;
+    struct ele_rec *eptr = current;
+
     current = current->next;
     eptr->next = NULL;
-    if (eptr->edata != NULL)
-    {
-      XtFree((char *) eptr->edata);
-    }
-    if (eptr->anchorHRef != NULL)
-    {
-      XtFree((char *) eptr->anchorHRef);
-    }
-    if (eptr->anchorName != NULL)
-    {
-      XtFree((char *) eptr->anchorName);
-    }
+    XtFree((char *) eptr->edata); /* accepts NULL... */
+    XtFree((char *) eptr->anchorHRef);
+    XtFree((char *) eptr->anchorName);
+    FreeIMGInfo (eptr->img_data);  /* also accepts NULL */
     XtFree((char *) eptr);
   }
 }
@@ -484,10 +392,11 @@ struct ele_rec *eptr;
 
 
 /*
- * Contruct and return an array of pointers into the element list that
+ * Construct and return an array of pointers into the element list that
  * indexes the elements by line number.
- * Note, lines containing only while space will have NULL pointers
+ * Note, lines containing only white space will have NULL pointers
  * into the element list.
+ * (FreeLineList and MakeLineList are completely unrelated functionally.)
  */
 struct ele_rec **
 MakeLineList (elist, max_line)
@@ -724,6 +633,10 @@ struct ref_rec *list;
 }
 
 
+/*
+ * The AddHRef/FindHRef mechanisms aren't used by Chimera
+ */
+#if 0
 /*
  * Find an element in the linked list of Internal HREFS.
  * return a pointer to the element, or NULL if not found.
@@ -782,86 +695,4 @@ char *href;
 
   return (list);
 }
-
-
-/*
- * Free up the allocated list of visited delayed images
- */
-void
-FreeDelayedImages (list)
-struct delay_rec *list;
-{
-  struct delay_rec *iptr;
-  struct delay_rec *tptr;
-
-  iptr = list;
-  while (iptr != NULL)
-  {
-    tptr = iptr;
-    iptr = iptr->next;
-    if (tptr->src != NULL)
-    {
-      XtFree((char *) tptr->src);
-    }
-    XtFree((char *) tptr);
-  }
-}
-
-
-/*
- * Find an element in the linked list of visited delayed images.
- * return a pointer to the element, or NULL if not found.
- */
-struct delay_rec *
-FindDelayedImage (list, src)
-struct delay_rec *list;
-char *src;
-{
-  struct delay_rec *iptr;
-
-  if (src == NULL)
-  {
-    return (NULL);
-  }
-
-  iptr = list;
-  while (iptr != NULL)
-  {
-    if (strcmp (iptr->src, src) == 0)
-    {
-      break;
-    }
-    iptr = iptr->next;
-  }
-  return (iptr);
-}
-
-
-/*
- * Add an element to the linked list of visited delayed images.
- * return a pointer to the head of the new list.
- */
-struct delay_rec *
-AddDelayedImage (list, src)
-struct delay_rec *list;
-char *src;
-{
-  struct delay_rec *iptr;
-
-  if (src == NULL)
-  {
-    return (list);
-  }
-
-  iptr = FindDelayedImage (list, src);
-
-  if (iptr == NULL)
-  {
-    iptr = (struct delay_rec *)XtMalloc(sizeof (struct delay_rec));
-    iptr->src = XtNewString(src);
-    iptr->next = list;
-    list = iptr;
-  }
-
-  return (list);
-}
+#endif /* not used */
