@@ -80,6 +80,7 @@ extern struct mark_up *AddObj ();
 /*
  * A hack to speed up caseless_equal.  Thanks to Quincey Koziol for
  * developing it for me
+ * Constraint: nothing but '\0' may map to 0
  */
 unsigned char map_table[256] =
 {
@@ -103,75 +104,31 @@ unsigned char map_table[256] =
 
 
 /*
- * Check if two strings are equal, ignoring case.
- * The strings must be of the same length to be equal.
- * return 1 if equal, 0 otherwise.
+ * Check if string s1 begins with s2, ignoring case (as defined by TOLOWER).
+ * To match, s1 must have either NUL or a !isalnum() char where s2 ends.
+ * Return 1 if s1 and s2 are non-NULL and s1 begins with s2; 0 otherwise.
  */
 int
-caseless_equal (str1, str2)
-char *str1;
-char *str2;
+caseless_equal (s1, s2)
+const unsigned char *s1;
+const unsigned char *s2;
 {
-  if ((str1 == NULL) || (str2 == NULL))
+  if (s1 == NULL  ||  s2 == NULL)
   {
     return (0);
   }
 
-  while ((*str1 != '\0') && (*str2 != '\0'))
+  while (*s2 != '\0')
   {
-    if (TOLOWER (*str1) != TOLOWER (*str2))
+    if (TOLOWER (*s1) != TOLOWER (*s2))	 /* true if *s1 == 0 ! */
     {
       return (0);
     }
-    str1++;
-    str2++;
+    s1++;
+    s2++;
   }
 
-  if ((*str1 == '\0') && (*str2 == '\0'))
-  {
-    return (1);
-  }
-  else
-  {
-    return (0);
-  }
-}
-
-
-/*
- * Check if two strings are equal in the first count characters, ignoring case.
- * The strings must both be at least of length count to be equal.
- * return 1 if equal, 0 otherwise.
- */
-int
-caseless_equal_prefix (str1, str2, cnt)
-char *str1;
-char *str2;
-int cnt;
-{
-  int i;
-
-  if ((str1 == NULL) || (str2 == NULL))
-  {
-    return (0);
-  }
-
-  if (cnt < 1)
-  {
-    return (1);
-  }
-
-  for (i = 0; i < cnt; i++)
-  {
-    if (TOLOWER (*str1) != TOLOWER (*str2))
-    {
-      return (0);
-    }
-    str1++;
-    str2++;
-  }
-
-  return (1);
+  return (*s1 == '\0'  ||  !isalnum(*s1)) ?  1 : 0;
 }
 
 
@@ -202,7 +159,7 @@ char *txt;
   }
 
   /*
-   * find a word, copying if we removed some space already
+   * Find a word, copying if we removed some space already
    */
   if (start == ptr)
   {
@@ -261,13 +218,10 @@ char *txt;
 
 
 /*
- * parse an amperstand escape, and return the appropriate character, or
+ * Parse an ampersand escape, and return the appropriate character, or
  * '\0' on error.
- * we should really only use caseless_equal_prefix for unterminated, and use
- * caseless_equal otherwise, but since there are so many escapes, and I
- * don't want to type everything twice, I always use caseless_equal_prefix
  * Turns out the escapes are case sensitive, use strncmp.
- * termination states:
+ * Termination states:
  *      0: terminated with a ';'
  *      1: unterminated
  *      2: terminated with whitespace
@@ -344,10 +298,10 @@ int termination;
  * "&amp;" = "&"
  * GAG:  apperantly &lt etc. can be left unterminated, what a nightmare.
  * Ok, better, they have to be terminated with white-space or ';'.
- * the '&' character must be immediately followed by a letter to be
+ * The '&' character must be immediately followed by a letter to be
  * a valid escape sequence.  Other &'s are left alone.
  * The cleaning is done by rearranging chars in the passed txt buffer.
- * if any escapes are replaced, the string becomes shorter.
+ * If any escapes are replaced, the string becomes shorter.
  */
 void
 clean_text (txt)
@@ -371,7 +325,7 @@ char *txt;
   /*
    * Quick scan to find escape sequences.
    * Escape is '&' followed by a letter (or a hash mark).
-   * return if there are none.
+   * Return if there are none.
    */
   ptr = txt;
   while (*ptr != '\0')
@@ -389,7 +343,7 @@ char *txt;
   }
 
   /*
-   * Loop, replaceing escape sequences, and moving up remaining
+   * Loop, replacing escape sequences, and moving up remaining
    * text.
    */
   ptr2 = ptr;
@@ -453,7 +407,7 @@ char *txt;
       space_terminated = 0;
     }
     /*
-     * invalid escape sequence. skip it.
+     * Invalid escape sequence. skip it.
      */
     else
     {
@@ -557,7 +511,7 @@ char **endp;
 /*
  * Get the mark text between '<' and '>'.  From the text, determine
  * its type, and fill in a mark_up structure to return.  Also returns
- * endp pointing to the ttrailing '>' in the original string.
+ * endp pointing to the trailing '>' in the original string.
  */
 struct mark_up *
 get_mark (start, endp)
@@ -581,15 +535,112 @@ char **endp;
 
   start++;
 
+  /*
+   * Parsing comments:
+   * <! comment >  is called a "short comment" below.
+   * <!-- comment --> is called a "long comment" below.
+   * <script> ... </script> is a comment, too. ;) -- GN 06Feb97
+   *
+   * Technically, the SGML spec says the "--" comment ends on the next "--",
+   * but that gives poor results on pages containing Java applets that use
+   * the autodecrement operator.
+   * If for some strange reason you really want SGML-spec style long comments,
+   * #define SGML_COMMENTS.  Otherwise <!-- ends on the next -->.
+   */
+
+  /* flush long or short comments outright */
+  if (*start == '!')
+  {
+    ptr = start + 1;
+#ifndef SGML_COMMENTS
+    if (*ptr == '-'  &&  ptr[1] == '-')  /* long comment */
+    {
+      /* search for a "-->" terminator */
+      for (ptr += 2;  *ptr != '\0';  ++ptr)
+      {
+	if (*ptr == '-'  &&  ptr[1] == '-'  &&  ptr[2] == '>')
+	{
+	  /* discard the comment text and pretend nothing happened */
+	  *endp = ptr + 2;
+	  return NULL;
+	}
+      }
+      /*
+       * Gak!  End of document reached without finding comment close!
+       * Rather than believe that, go back and use short comment rules.
+       */
+      ptr = start + 1;
+    }
+    /* next ">" terminates comment */
+    while (*ptr != '>'  &&  *ptr != '\0')   ptr++;
+
+#else  /* want SGML_COMMENTS */
+    {
+      int is_comment = 0;
+      while ((is_comment || (*ptr != '>')) && (*ptr != '\0'))
+      {				/* keep track of '--' bracketed comments */
+	if ((*ptr == '-') && (*++ptr == '-'))
+	{
+	  is_comment = 1 - is_comment;
+	}
+	ptr++;
+      }
+      /*
+       * Theoretically, an SGML Comment Declaration contains ONLY zero or more
+       * '--' delimited comments, but we aren't picky about finding other stuff
+       * between comments
+       */
+    }
+#endif /*ifndef SGML_COMMENTS*/
+
+    *endp = ptr;
+    return NULL;
+  }
+
   mark = (struct mark_up *)XtMalloc(sizeof (struct mark_up));
 
   /*
    * Grab the mark text
    */
   ptr = start;
-  while ((*ptr != '>') && (*ptr != '\0'))
+
+  /*
+   * Handle <script> ... </script> comments
+   * It would have been nice to flush the text outright, but the calling
+   * code doesn't understand returning NULL with start[1] != '!'.  So,
+   * waste time getting a mark_up struct, saving the text, and discarding it.
+   */
+
+  if (caseless_equal (ptr, "script")  &&  isspace (ptr[6]) )
+  {				/* this is a script el't. Treat as comment */
+    int is_comment = 1;
+    ptr += 7;
+    while ((is_comment || (*ptr != '>')) && (*ptr != '\0'))
+    {
+      if ((*ptr == '<') && (*++ptr == '/')
+	  && (caseless_equal (++ptr, "script")))
+      {
+	ptr += 6;
+	if (*ptr == '>')
+	  break;
+	else if (isspace(*ptr))
+	{
+	  is_comment = 0;
+	  ptr++;
+	}
+      }
+      else
+      {
+	ptr++;
+      }
+    }
+  }
+  else				/* ordinary SGML tag */
   {
-    ptr++;
+    while ((*ptr != '>') && (*ptr != '\0'))
+    {
+      ptr++;
+    }
   }
   *endp = ptr;
 
@@ -778,7 +829,7 @@ char *str;
   {
     /*
      * Get some text (if any).  If our last mark was
-     * a begin plain text we call different function
+     * a begin plain text, we call a different function.
      * If last mark was <PLAINTEXT> we lump all the rest of
      * the text in.
      */
@@ -817,7 +868,9 @@ char *str;
     {
       if (start != end)
       {
+#ifdef VERBOSE
 	fprintf (stderr, "error parsing text, bailing out\n");
+#endif
 	return (list);
       }
     }
@@ -846,60 +899,26 @@ char *str;
     mark = get_mark (start, &end);
     if (mark == NULL)
     {
-      if (start != end)
+      if (start != end  &&  start[1] != '!')
       {
+#ifdef VERBOSE
 	fprintf (stderr, "error parsing mark, bailing out\n");
+#endif
 	return (list);
       }
+      start = end + 1;
     }
     else
     {
+      start = end + 1;
+
       mark->next = NULL;
       current = AddObj (&list, current, mark, preformat);
-    }
 
-    start = (char *) (end + 1);
-
-    if ((mark != NULL) && (mark->type == M_PLAIN_FILE) &&
-	(!mark->is_end))
-    {
-      /*
-       * A linefeed immediately after the <PLAINTEXT>
-       * mark is to be ignored.
-       */
-      if (*start == '\n')
+      if ((mark->type == M_PLAIN_FILE)  &&  (!mark->is_end))
       {
-	start++;
-      }
-    }
-    else if ((mark != NULL) && ((mark->type == M_PLAIN_TEXT) ||
-				(mark->type == M_LISTING_TEXT)) &&
-	     (!mark->is_end))
-    {
-      /*
-       * A linefeed immediately after the <XMP>
-       * or <LISTING> mark is to be ignored.
-       */
-      if (*start == '\n')
-      {
-	start++;
-      }
-    }
-    /*
-     * If we are parsing pre-formatted text we need to set a
-     * flag so we don't throw out needed linefeeds.
-     */
-    else if ((mark != NULL) && (mark->type == M_PREFORMAT))
-    {
-      if (mark->is_end)
-      {
-	preformat = 0;
-      }
-      else
-      {
-	preformat = 1;
 	/*
-	 * A linefeed immediately after the <PRE>
+	 * A linefeed immediately after the <PLAINTEXT>
 	 * mark is to be ignored.
 	 */
 	if (*start == '\n')
@@ -907,6 +926,41 @@ char *str;
 	  start++;
 	}
       }
+      else if ((mark->type == M_PLAIN_TEXT  ||  mark->type == M_LISTING_TEXT)
+	       && (!mark->is_end))
+      {
+	/*
+	 * A linefeed immediately after the <XMP>
+	 * or <LISTING> mark is to be ignored.
+	 */
+	if (*start == '\n')
+	{
+	  start++;
+	}
+      }
+      /*
+       * If we are parsing pre-formatted text we need to set a
+       * flag so we don't throw out needed linefeeds.
+       */
+      else if (mark->type == M_PREFORMAT)
+      {
+	if (mark->is_end)
+	{
+	  preformat = 0;
+	}
+	else
+	{
+	  preformat = 1;
+	  /*
+	   * A linefeed immediately after the <PRE>
+	   * mark is to be ignored.
+	   */
+	  if (*start == '\n')
+	  {
+	    start++;
+	  }
+	}
+      }
     }
   }
 #ifdef TIMING
@@ -925,15 +979,20 @@ ParseMarkType (str)
 char *str;
 {
   int type;
-  char *tptr;
-  char tchar;
+/*  unsigned char *tptr;
+/*  char tchar;*/
 
   if (str == NULL)
   {
     return (M_NONE);
   }
 
-  type = M_UNKNOWN;
+  /*
+   * On the reasonable assumptions that our MT_* strings don't have
+   * NUL or space characters in them, and that the caller stripped leading
+   * spaces (not strictly necessary), this next chunk of code is useless.
+   */
+#if 0
   tptr = str;
   while (*tptr != '\0')
   {
@@ -945,6 +1004,7 @@ char *str;
   }
   tchar = *tptr;
   *tptr = '\0';
+#endif
 
   if (caseless_equal (str, MT_ANCHOR))
   {
@@ -1122,6 +1182,26 @@ char *str;
   {
     type = M_FORM;
   }
+  else if (caseless_equal (str, MT_TABLE))
+    {
+    type = M_TABLE;
+    }
+  else if (caseless_equal (str, MT_TABLE_ITEM))
+    {
+    type = M_TABLE_ITEM;
+    }
+  else if (caseless_equal (str, MT_TABLE_HEADER))
+    {
+    type = M_TABLE_HEADER;
+    }
+  else if (caseless_equal (str, MT_TABLE_ROW))
+    {
+    type = M_TABLE_ROW;
+    }
+  else if (caseless_equal (str, MT_CAPTION))
+  {
+    type = M_CAPTION;
+  }
   else
   {
 #ifdef VERBOSE
@@ -1130,7 +1210,8 @@ char *str;
     type = M_UNKNOWN;
   }
 
-  *tptr = tchar;
+/*  *tptr = tchar;*/
+
   return (type);
 }
 
@@ -1145,14 +1226,13 @@ char *str;
  */
 char *
 AnchorTag (ptrp, startp, endp)
-char **ptrp;
-char **startp;
-char **endp;
+const char **ptrp;
+const char **startp;
+const char **endp;
 {
   char *tag_val;
-  char *ptr;
-  char *start;
-  char tchar;
+  const char *ptr;
+  const char *start;
   int quoted;
   int has_value;
 
@@ -1210,11 +1290,11 @@ char **endp;
     return (XtNewString("1"));
   }
 
-  if (*ptr == '\"')
-  {
-    quoted = 1;
-    ptr++;
-  }
+  /*
+   * Single quotes are also legal SGML value delimiters --GN 1997May06
+   */
+  if (*ptr == '\"' || *ptr == '\'') 
+    quoted = *ptr++;		/* remember which one it was */
 
   start = ptr;
   /*
@@ -1222,7 +1302,7 @@ char **endp;
    */
   if (quoted)
   {
-    while ((*ptr != '\"') && (*ptr != '\0'))
+    while ((*ptr != quoted) && (*ptr != '\0'))
     {
       ptr++;
     }
@@ -1241,12 +1321,14 @@ char **endp;
   }
 
   /*
-   * Copy the tag value out into a malloced string
+   * Copy the tag value out into a malloc'd string.
+   * Logically XtNewString, but avoids modifying the string passed by the
+   * caller (which might be unwritable).
    */
-  tchar = *ptr;
-  *ptr = '\0';
-  tag_val = XtNewString(start);
-  *ptr = tchar;
+  tag_val = XtMalloc (ptr - start + 1);
+  tag_val[0] = '\0';		/* for strncat */
+  strncat (tag_val, start, ptr - start);  /* should work even if ptr=start */
+
   if (quoted) ptr++;
   *ptrp = ptr;
 
@@ -1262,37 +1344,39 @@ char **endp;
  */
 char *
 ParseMarkTag (text, mtext, mtag)
-char *text;
-char *mtext;
-char *mtag;
+const char *text;
+const char *mtext;
+const char *mtag;
 {
   char *ptr;
-  char *start;
-  char *end;
-  char *tag_val;
-  char tchar;
 
   if ((text == NULL) || (mtext == NULL) || (mtag == NULL))
   {
     return (NULL);
   }
 
+  /*
+   * skip over leading mark tag (mtext) characters.
+   * Special cases: common practice is to pass MT_tag, which doesn't
+   * include the leading "/" in closing marks and which doesn't include the
+   * leading "+" in certain implicitly-generated tags.  Skip them both.
+   */
+  if (*text == '+')  ++text;	/* always first, if present */
+  if (*text == '/')  ++text;
   ptr = (char *) (text + strlen (mtext));
 
   while (*ptr != '\0')
   {
-    tag_val = AnchorTag (&ptr, &start, &end);
+    char *start, *end;
 
-    tchar = *end;
-    *end = '\0';
-    if (caseless_equal (start, mtag))
+    char *tag_val = AnchorTag (&ptr, &start, &end);
+
+    if (caseless_equal (start, mtag))  /* found the requested field in tag */
     {
-      *end = tchar;
-      if (tag_val == NULL) return (XtNewString(""));
-      else return (tag_val);
+      return (tag_val != NULL) ?  tag_val  :  XtNewString("");
     }
-    *end = tchar;
-    if (tag_val != NULL) XtFree(tag_val);
+    if (tag_val != NULL)  XtFree (tag_val);
   }
+
   return (NULL);
 }
