@@ -1,3 +1,5 @@
+/* substantially modified by WBE, Spring 1997, and by others before me */
+
 /****************************************************************************
  * NCSA Mosaic for the X Window System                                      *
  * Software Development Group                                               *
@@ -175,8 +177,8 @@ XtPointer call_data;
   int item_count;
 
   XtVaGetValues (w,
-		 XtNlist, string_list,
-		 XtNnumberStrings, item_count,
+		 XtNlist, &string_list,
+		 XtNnumberStrings, &item_count,
 		 NULL);
 
   p = string_list;
@@ -213,6 +215,7 @@ XtPointer call_data;
   XtVaSetValues (menuButton, XtNlabel, label, NULL);
 }
 
+
 void
 AddNewForm (hw, fptr)
 HTMLWidget hw;
@@ -220,11 +223,12 @@ FormInfo *fptr;
 {
   FormInfo *ptr;
 
+  fptr->next = NULL;
+
   ptr = hw->html.form_list;
   if (ptr == NULL)
   {
     hw->html.form_list = fptr;
-    fptr->next = NULL;
   }
   else
   {
@@ -233,11 +237,39 @@ FormInfo *fptr;
       ptr = ptr->next;
     }
     ptr->next = fptr;
-    fptr->next = NULL;
   }
 }
 
 
+void
+FreeOneForm (fptr)
+FormInfo *fptr;
+{
+  if (fptr->action != NULL)      XtFree (fptr->action);
+  if (fptr->method != NULL)      XtFree (fptr->method);
+  if (fptr->enctype != NULL)     XtFree (fptr->enctype);
+  if (fptr->enc_entity != NULL)  XtFree (fptr->enc_entity);
+  XtFree ((char *)fptr);
+}
+
+
+void
+FreeForms (hw)
+HTMLWidget hw;
+{
+  FormInfo *fptr;
+
+  fptr = hw->html.form_list;
+  while (fptr != NULL)
+  {
+    FormInfo *next = fptr->next;
+    FreeOneForm (fptr);
+    fptr = next;
+  }
+  hw->html.form_list = NULL;
+}
+
+
 int
 CollectSubmitInfo (w, fptr, name_list, value_list)
 Widget w;
@@ -438,7 +470,7 @@ char ***value_list;
 	  cbdata.attribute_values[cnt] = wptr->value;
 	  break;
 	case W_PUSHBUTTON:
-	  cbdata.attribute_values[cnt] = NULL;
+	  cbdata.attribute_values[cnt] = wptr->value; /* SKLindsay 240796 */
 	  break;
         default:
 	  cbdata.attribute_values[cnt] = NULL;
@@ -905,11 +937,12 @@ XtPointer call_data;
 	    
 	    if (vlist_cnt > 0)
 	    {
+#ifdef VERBOSE
 	      if (vlist_cnt > 1)
 	      {
 		fprintf (stderr, "HTML: only a single selection allowed!\n");
 	      }
-	      
+#endif
 	      for (i = 0; i < list_cnt; i++)
 	      {
 		if (!strcasecmp (string_list[i], val_list[0]))
@@ -1065,9 +1098,8 @@ char *name;
 }
 
 WidgetInfo *
-AddNewWidget (hw, fptr, w, type, id, x, y, name, value, mapping, checked)
+AddNewWidget (hw, w, type, id, x, y, name, value, mapping, checked)
 HTMLWidget hw;
-FormInfo *fptr;
 Widget w;
 int type;
 int id;
@@ -1100,48 +1132,33 @@ Boolean checked;
     height = 0;
   }
 
-  wptr = hw->html.widget_list;
-  if (wptr == NULL)
+  wptr = (WidgetInfo *) XtMalloc (sizeof (WidgetInfo));
+  wptr->w = w;
+  wptr->type = type;
+  wptr->id = id;
+  wptr->x = x;
+  wptr->y = y;
+  wptr->width = width;
+  wptr->height = height;
+  wptr->name = name;
+  wptr->value = value;
+  wptr->password = NULL;
+  wptr->mapping = mapping;
+  wptr->checked = checked;
+  wptr->mapped = False;
+  wptr->next = NULL;
+
+  if (hw->html.widget_list == NULL)
   {
-    wptr = (WidgetInfo *) XtMalloc (sizeof (WidgetInfo));
-    wptr->w = w;
-    wptr->type = type;
-    wptr->id = id;
-    wptr->x = x;
-    wptr->y = y;
-    wptr->width = width;
-    wptr->height = height;
-    wptr->name = name;
-    wptr->value = value;
-    wptr->password = NULL;
-    wptr->mapping = mapping;
-    wptr->checked = checked;
-    wptr->mapped = False;
-    wptr->next = NULL;
     hw->html.widget_list = wptr;
   }
-  else
+  else				/* find tail */
   {
-    while (wptr->next != NULL)
-    {
-      wptr = wptr->next;
-    }
-    wptr->next = (WidgetInfo *) XtMalloc (sizeof (WidgetInfo));
-    wptr = wptr->next;
-    wptr->w = w;
-    wptr->type = type;
-    wptr->id = id;
-    wptr->x = x;
-    wptr->y = y;
-    wptr->width = width;
-    wptr->height = height;
-    wptr->name = name;
-    wptr->value = value;
-    wptr->password = NULL;
-    wptr->mapping = mapping;
-    wptr->checked = checked;
-    wptr->mapped = False;
-    wptr->next = NULL;
+    WidgetInfo *tail = hw->html.widget_list;
+
+    while (tail->next != NULL)   tail = tail->next;
+
+    tail->next = wptr;
   }
 
   if ((wptr->type == W_PASSWORD) && (wptr->value != NULL))
@@ -1153,7 +1170,7 @@ Boolean checked;
 }
 
 /*
- * For the various widgets, return their fon structures so
+ * For the various widgets, return their font structures so
  * we can use the font's baseline to place them.
  */
 XFontStruct *
@@ -1518,7 +1535,7 @@ HTMLWidget hw;
 char *text;
 int x, y;
 int id;
-FormInfo *fptr;
+FormInfo *fptr;			/* not used */
 {
   Widget w;
   char **list;
@@ -1589,8 +1606,7 @@ FormInfo *fptr;
   XtSetMappedWhenManaged (w, False);
   XtManageChild (w);
 
-  return (AddNewWidget (hw, fptr, w, W_TEXTAREA, id, x, y,
-			name, value, NULL, 0));
+  return (AddNewWidget (hw, w, W_TEXTAREA, id, x, y, name, value, NULL, 0));
 }
 
 /*
@@ -1603,7 +1619,7 @@ HTMLWidget hw;
 char *text;
 int x, y;
 int id;
-FormInfo *fptr;
+FormInfo *fptr;			/* not used */
 {
   char **list;
   int list_cnt;
@@ -1658,7 +1674,7 @@ FormInfo *fptr;
   if (value == NULL) value = XtNewString ("");
 
   /*
-   * For textfileds parse maxlength and
+   * For textfields parse maxlength and
    * set up the widget.
    */
   if (type == W_TEXTFIELD)
@@ -1721,8 +1737,7 @@ FormInfo *fptr;
   XtSetMappedWhenManaged (w, False);
   XtManageChild (w);
 
-  return (AddNewWidget (hw, fptr, w, type, id, x, y,
-			name, value, NULL, False));
+  return (AddNewWidget (hw, w, type, id, x, y, name, value, NULL, False));
 }
 
 static WidgetInfo *
@@ -1732,7 +1747,7 @@ HTMLWidget hw;
 char *text;
 int x, y;
 int id;
-FormInfo *fptr;
+FormInfo *fptr;			/* not used */
 {
   Arg arg[30];
   int size, maxlength;
@@ -1790,7 +1805,7 @@ FormInfo *fptr;
   XtSetMappedWhenManaged (w, False);
   XtManageChild (w);
 
-  return (AddNewWidget (hw, fptr, w, W_PASSWORD, id, x, y,
+  return (AddNewWidget (hw, w, W_PASSWORD, id, x, y,
 			name, value, NULL, False));
 }
 
@@ -1804,7 +1819,7 @@ HTMLWidget hw;
 char *text;
 int x, y;
 int id;
-FormInfo *fptr;
+FormInfo *fptr;			/* not used */
 {
   Boolean checked;
   char *value;
@@ -1835,7 +1850,7 @@ FormInfo *fptr;
   XtSetMappedWhenManaged (w, False);
   XtManageChild (w);
 
-  return (AddNewWidget (hw, fptr, w, W_CHECKBOX, id, x, y,
+  return (AddNewWidget (hw, w, W_CHECKBOX, id, x, y,
 			name, value, NULL, checked));
 }
 
@@ -1849,14 +1864,14 @@ HTMLWidget hw;
 char *text;
 int x, y;
 int id;
-FormInfo *fptr;
+FormInfo *fptr;			/* not used */
 {
   char *value;
 
   value = ParseMarkTag (text, MT_INPUT, "VALUE");
   if (value == NULL) XtNewString("");
 
-  return (AddNewWidget (hw, fptr, NULL, W_HIDDEN, id, x, y,
+  return (AddNewWidget (hw, NULL, W_HIDDEN, id, x, y,
 			name, value, NULL, False));
 }
 
@@ -1912,7 +1927,7 @@ FormInfo *fptr;
   XtSetMappedWhenManaged (w, False);
   XtManageChild (w);
 
-  return (AddNewWidget (hw, fptr, w, W_RADIOBOX, id, x, y,
+  return (AddNewWidget (hw, w, W_RADIOBOX, id, x, y,
 			name, value, NULL, checked));
 }
 
@@ -1953,7 +1968,7 @@ FormInfo *fptr;
   XtManageChild (w);
   PrepareFormEnd (hw, w, fptr);
 
-  return (AddNewWidget (hw, fptr, w, W_PUSHBUTTON, id, x, y,
+  return (AddNewWidget (hw, w, W_PUSHBUTTON, id, x, y,
 			name, value, NULL, False));
 }
 
@@ -1967,7 +1982,7 @@ HTMLWidget hw;
 char *text;
 int x, y;
 int id;
-FormInfo *fptr;
+FormInfo *fptr;			/* not used */
 {
   int type;
   Widget w;
@@ -2231,11 +2246,12 @@ FormInfo *fptr;
 
     if (vlist_cnt > 0)
     {
+#ifdef VERBOSE
       if (vlist_cnt > 1)
       {
 	fprintf (stderr, "HTML: only a single selection allowed!\n");
       }
-
+#endif
       for (i = 0; i < list_cnt; i++)
       {
 	if (!strcasecmp (string_list[i], val_list[0]))
@@ -2276,7 +2292,7 @@ FormInfo *fptr;
   XtSetMappedWhenManaged (w, False);
   XtManageChild (w);
 
-  return (AddNewWidget (hw, fptr, w, type, id, x, y,
+  return (AddNewWidget (hw, w, type, id, x, y,
 			name, value, mapping, False));
 }
 
@@ -2317,7 +2333,7 @@ FormInfo *fptr;
   XtManageChild (w);
   PrepareFormReset (hw, w, fptr);
 
-  return (AddNewWidget (hw, fptr, w, W_PUSHBUTTON, id, x, y,
+  return (AddNewWidget (hw, w, W_PUSHBUTTON, id, x, y,
 			name, value, NULL, False));
 }
 
@@ -2331,7 +2347,7 @@ HTMLWidget hw;
 char *text;
 int x, y;
 int id;
-FormInfo *fptr;
+FormInfo *fptr;			/* not used */
 {
   char *value;
   Cardinal argcnt;
@@ -2353,10 +2369,39 @@ FormInfo *fptr;
   XtSetMappedWhenManaged (w, False);
   XtManageChild (w);
 
-  return (AddNewWidget (hw, fptr, w, W_PUSHBUTTON, id, x, y,
+  return (AddNewWidget (hw, w, W_PUSHBUTTON, id, x, y,
 			name, value, NULL, False));
 }
 
+/*
+ * Move the widget to a new x,y location and update the WidgetInfo x,y.
+ */
+void
+MoveWidget (wp, x, y)
+WidgetInfo *wp;
+int x,y;
+{
+    wp->x = x;
+    wp->y = y;
+
+    /*
+     * Don't want to SetValues if type HIDDEN which has no widget.
+     */
+    if (wp->w != NULL)
+    {
+      Arg arg[10];
+      Cardinal argcnt;
+
+      XtUnmanageChild (wp->w);
+      argcnt = 0;
+      XtSetArg (arg[argcnt], XtNx, x); argcnt++;
+      XtSetArg (arg[argcnt], XtNy, y); argcnt++;
+      XtSetValues (wp->w, arg, argcnt);
+      XtManageChild (wp->w);
+    }
+}
+
+
 /*
  * Make the appropriate widget for this tag, and fill in an
  * WidgetInfo structure and return it.
@@ -2372,12 +2417,10 @@ FormInfo *fptr;
   WidgetInfo *wlist;
   WidgetInfo *wptr;
 
-  wlist = hw->html.widget_list;
-  while (wlist != NULL)
+  for (wlist = hw->html.widget_list;  wlist != NULL;  wlist = wlist->next)
   {
     if (wlist->id == id)
       break;
-    wlist = wlist->next;
   }
 
   /*
@@ -2439,29 +2482,10 @@ FormInfo *fptr;
   else
     /*
      * We found this widget on the list of already created widgets.
-     * Put it in place for reuse.
+     * Move it to its new location.
      */
   {
-    wlist->x = x;
-    wlist->y = y;
-
-    /*
-     * Don't want to SetValues if type HIDDEN which
-     * has no widget.
-     */
-    if (wlist->w != NULL)
-    {
-      Arg arg[10];
-      Cardinal argcnt;
-
-      XtUnmanageChild (wlist->w);
-      argcnt = 0;
-      XtSetArg (arg[argcnt], XtNx, x); argcnt++;
-      XtSetArg (arg[argcnt], XtNy, y); argcnt++;
-      XtSetValues (wlist->w, arg, argcnt);
-      XtManageChild (wlist->w);
-    }
-
+    MoveWidget (wlist, x, y);
     wptr = wlist;
   }
 
@@ -2480,74 +2504,4 @@ struct ele_rec *eptr;
     eptr->widget_data->mapped = True;
     XtSetMappedWhenManaged (eptr->widget_data->w, True);
   }
-
-#if 0
-  if (eptr->pic_data != NULL)
-  {
-    int x, y, extra;
-
-    x = eptr->x;
-    y = eptr->y + eptr->y_offset;
-
-    if ((hw->html.border_images == True) ||
-	(eptr->anchorHRef != NULL))
-    {
-      extra = IMAGE_BORDER;
-    }
-    else
-    {
-      extra = 0;
-    }
-
-    x = x - hw->html.scroll_x;
-    y = y - hw->html.scroll_y;
-    XSetForeground (XtDisplay (hw), hw->html.drawGC, eptr->fg);
-    XSetBackground (XtDisplay (hw), hw->html.drawGC, eptr->bg);
-    XFillRectangle (XtDisplay (hw->html.view),
-		    XtWindow (hw->html.view), hw->html.drawGC,
-		    x, y,
-		    (eptr->pic_data->width + (2 * extra)),
-		    extra);
-    XFillRectangle (XtDisplay (hw->html.view),
-		    XtWindow (hw->html.view), hw->html.drawGC,
-		    x,
-		    (y + eptr->pic_data->height + extra),
-		    (eptr->pic_data->width + (2 * extra)),
-		    extra);
-    XFillRectangle (XtDisplay (hw->html.view),
-		    XtWindow (hw->html.view), hw->html.drawGC,
-		    x, y,
-		    extra,
-		    (eptr->pic_data->height + (2 * extra)));
-    XFillRectangle (XtDisplay (hw->html.view),
-		    XtWindow (hw->html.view), hw->html.drawGC,
-		    (x + eptr->pic_data->width + extra),
-		    y,
-		    extra,
-		    (eptr->pic_data->height + (2 * extra)));
-
-    if (eptr->pic_data->image == (Pixmap) NULL)
-    {
-      if (eptr->pic_data->image_data != NULL)
-      {
-	eptr->pic_data->image = InfoToImage (hw,
-					     eptr->pic_data);
-      }
-      else
-      {
-	eptr->pic_data->image = NoImage (hw);
-      }
-    }
-
-    if (eptr->pic_data->image != (Pixmap) NULL)
-    {
-      XCopyArea (XtDisplay (hw->html.view),
-		 eptr->pic_data->image,
-		 XtWindow (hw->html.view), hw->html.drawGC, 0, 0,
-		 eptr->pic_data->width, eptr->pic_data->height,
-		 (x + extra),
-		 (y + extra));
-    }
-  }
-#endif
 }
