@@ -311,10 +311,12 @@ Widget w;
   BookmarkGroup *bmg, *tmg;
 
   if (BREC.delayWrite) WriteBookmarkFile((BookmarkWidget)w);
+
+  if (BREC.filename != NULL)     XtFree(BREC.filename);
   if (BREC.currentGroup != NULL) XtFree(BREC.currentGroup);
-  if (BREC.nexturl != NULL) XtFree(BREC.nexturl);
-  if (BREC.bgslist != NULL) XtFree((char *)BREC.bgslist);
-  if (BREC.bslist != NULL) XtFree((char *)BREC.bslist);
+  if (BREC.nexturl != NULL)      XtFree(BREC.nexturl);
+  if (BREC.bgslist != NULL)      XtFree((char *)BREC.bgslist);
+  if (BREC.bslist != NULL)       XtFree((char *)BREC.bslist);
   
   for (bmg = BREC.blist; bmg != NULL; )
   {
