@@ -24,6 +24,8 @@ typedef struct _document
   int from_cache;       /* 1 if fetched from the cache */
   int cache;            /* 1 if the document should be cached */
   int status;           /* document status */
+  int view_frag;	/* 1 initially to jump to an <A NAME=>, 0 later
+			   to honor remembered scrollbar positions */
   char *auth_realm;     /* authentication realm */
   char *auth_type;      /* authentication type */
   MIMEField *mflist;    /* list of MIME fields for document */
