@@ -25,6 +25,7 @@
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 #include <X11/cursorfont.h>
+#include <X11/Xaw/Toggle.h>	/* for DeferPix (WBE) */
 
 #include "StrReq.h"
 #include "AuthReq.h"
@@ -53,14 +54,14 @@
 #include "widget.h"
 
 /*
- * convenience routines.  the ifdef is bogus.
+ * convenience routines.
  */
 static Document *LoadDoc _ArgProto((URLParts *, int, int));
 static void HandleDoc _ArgProto((Document *, int));
 static void AddMessage _ArgProto((char *));
 static void DisplayCurrent _ArgProto((void));
 static char *GetTitle _ArgProto((Widget));
-static void SetTitle _ArgProto((char *));
+/*static*/ void SetTitle _ArgProto((char *));
 static void SetURL _ArgProto((URLParts *));
 static void sigigh_handler _ArgProto(());
 static void SaveDocument _ArgProto((Document *, char *, int));
@@ -76,7 +77,8 @@ static void CreateStrReq _ArgProto((char *, char *, int, XtCallbackProc, XtCallb
 static void WarningHandler _ArgProto((String));
 static void ErrorHandler _ArgProto((String));
 
-int DisplayTransferStatus _ArgProto((int, int));
+int DisplayTransferStatus _ArgProto((int, int, int));
+int DisplayConnectStatus _ArgProto((int, char *));
 
 static void           HomeAction();
 static void           BackAction();
@@ -88,11 +90,12 @@ void                  OpenAction();
 void                  FileAction();
 void                  SearchAction();
 void                  BookmarkAction();
+static void	      DeferPixAction();
 static void           OpenURL();
 
 static AppResources root;
 
-#define BUTTON_LIST "quit, open, home, back, source, reload, file, help, bookmark, search, cancel"
+#define BUTTON_LIST "quit, open, home, back, source, reload, file, help, bookmark, search, cancel, deferpix"
 
 static char defaultTranslations[] =
 "\
@@ -106,6 +109,7 @@ static char defaultTranslations[] =
 <Key>f: file()\n\
 <Key>s: search()\n\
 <Key>m: bookmark()\n\
+<Key>i: deferpix()\n\
 ";
 
 #define offset(field) XtOffset(AppResources *, field)
@@ -121,6 +125,8 @@ static XtResource       resource_list[] =
 	offset(showURL), XtRImmediate, (XtPointer)True },
   { "showTitle", XtCBoolean, XtRBoolean, sizeof(Boolean),
 	offset(showTitle), XtRImmediate, (XtPointer)True },
+  { "anchorDisplay", XtCBoolean, XtRBoolean, sizeof(Boolean),
+       offset(anchorDisplay), XtRImmediate, (XtPointer)False },
   { "button1Box", "BoxList", XtRString, sizeof(char *),
 	offset(button1Box), XtRString, (XtPointer)BUTTON_LIST },
   { "button2Box", "BoxList", XtRString, sizeof(char *),
@@ -183,6 +189,10 @@ static XtResource       resource_list[] =
 	offset(cacheIgnoreExpires), XtRImmediate, (XtPointer)False },
   { "gammaCorrect", "GammaCorrect", XtRFloat, sizeof(float),
 	offset(gamma), XtRImmediate, (XtPointer)0 },
+  { "openButtonShortcut", XtCBoolean, XtRBoolean, sizeof(Boolean),
+	offset(openButtonShortcut), XtRImmediate, (XtPointer)True },
+  { "localIndexFiles", "LocalIndexFiles", XtRString, sizeof(char *),
+	offset(localIndexFiles), XtRString, (XtPointer)"index.html" },
 };
 
 static XtActionsRec actionsList[] =
@@ -198,6 +208,7 @@ static XtActionsRec actionsList[] =
   { "search",   (XtActionProc) SearchAction },
   { "bookmark", (XtActionProc) BookmarkAction },
   { "openurl",  (XtActionProc) OpenURL },
+  { "deferpix",	(XtActionProc) DeferPixAction },
 };
 
 extern char *fallback_resources[];
@@ -233,12 +244,12 @@ char **argv;
   /*
    * Huh?  This is goofy.  We are trying to trap a bunch of signals
    * in case the user has set cleanUp to True so that cache files can
-   * be cleaned.
+   * be cleaned.--- SIGQUIT should never be caught. [Jim Rees & GN]
    */
   signal(SIGINT, sigigh_handler);
-  signal(SIGQUIT, sigigh_handler);
   signal(SIGHUP, sigigh_handler);
   signal(SIGTERM, sigigh_handler);
+  signal(SIGALRM, SIG_IGN);	/* changed when necessary in src/net.c */
   signal(SIGPIPE, SIG_IGN);
 
   StartReaper();
@@ -320,6 +331,7 @@ char **argv;
   root.mclist = ReadMailCapFiles(root.mailCapFiles);
   root.mtlist = ReadMIMETypeFiles(root.mimeTypeFiles);
   root.plist = ReadProtocolFiles(root.protocolFiles);
+  ReadIndexFilenames(root.localIndexFiles);
 
   AddLanguage(root.languageDB);
 
@@ -465,6 +477,10 @@ XtInputId *xid;
 
 /*
  * GetTitle
+ *
+ * Extracts the title from the current HTML widget if it has one.
+ *
+ * ... and while we're at it, get rid of some annoying extra spaces.--GN
  */
 static char *
 GetTitle(hw)
@@ -477,10 +493,11 @@ Widget hw;
   if (t == NULL) s = alloc_string("");
   else
   {
+    while (*t && isspace(*t)) t++; /* skip leading whitespace */
     s = alloc_string(t);
     for (t = s; *t; t++)
     {
-      if (isspace(*t)) *t = ' ';
+      if (isspace(*t)) *t = ' '; /* normalize any further whitespace */
     }
   }
   return(s);
@@ -490,9 +507,10 @@ Widget hw;
  * SetTitle
  *
  * Sets the string in the title widget.  Passing a NULL as a string
- * makes the title of the current document the title.
+ * makes the title of the current document the title.  If it doesn't
+ * have a title, keep the last message around [GN].
  */
-static void
+/*static*/ void
 SetTitle(title)
 char *title;
 {
@@ -502,7 +520,8 @@ char *title;
   {
     if (title == NULL) title = t = GetTitle(root.w);
 
-    XtVaSetValues(root.titledisplay, XtNstring, title, NULL);
+    if (title[0])
+      XtVaSetValues(root.titledisplay, XtNstring, title, NULL);
 
     if (t != NULL) free_mem(t);
 
@@ -540,7 +559,8 @@ URLParts *up;
  * DisplayCurrent
  *
  * Displays the current HTML screen.  It also changes the title display
- * and the URL display.
+ * and the URL display.-- However, in certain error situations we prefer
+ * to leave the last status message in the title display untouched.
  */
 static void
 DisplayCurrent()
@@ -549,6 +569,7 @@ DisplayCurrent()
   DocNode *t;
   Arg args[2];
   Widget view;
+  char *displaying = NULL;	/* as in DisplayTransferStatus --GN */
 
   if (root.dlist == NULL)
   {
@@ -557,7 +578,8 @@ DisplayCurrent()
   }
 
   XDefineCursor(XtDisplay(root.toplevel), XtWindow(root.toplevel), root.watch);
-  
+  if (displaying == NULL) displaying = GetFromStringDB("display");
+
   /*
    * Give the new scrollbar position.
    */
@@ -565,7 +587,6 @@ DisplayCurrent()
   XtSetArg(args[1], WbNhorizontalScrollBarPos, root.dlist->hpos);
   XtSetValues(root.w, args, 2);
 
-  SetTitle(GetFromStringDB("display"));
   SetURL(root.dlist->up);
 
   /*
@@ -588,22 +609,24 @@ DisplayCurrent()
       root.dlist = root.dlist->next;
       if (t->base != NULL) free_mem(t->base);
       free_mem((char *)t);
-      DisplayCurrent();
+      DisplayCurrent();		/* recursive call */
       return;
     }
   }
+  if (d == NULL || d->status != DS_ERROR)
+    SetTitle(displaying);
 
-  HTMLFreeImageInfo(root.w);
+  /*DiscardCurPageObjects (root.w, root.w);*/ /* done by HTMLSetText */
 
   root.cancelop = False;
 
-  if (d->pcontent != NULL && strcasecmp(d->pcontent, "text/html") == 0)
+  if (d->pcontent != NULL  &&  strcasecmp(d->pcontent, "text/html") == 0)
   {
-    HTMLSetText(root.w, d->ptext, NULL, NULL, 0, NULL, NULL);
+    HTMLSetText(root.w, d->ptext, NULL, NULL);
   }
   else
   {
-    HTMLSetText(root.w, d->text, NULL, NULL, 0, NULL, NULL);
+    HTMLSetText(root.w, d->text, NULL, NULL);
   }
 
   if (d->up != NULL && d->up->anchor != NULL)
@@ -639,22 +662,21 @@ DisplayCurrent()
     if (root.home != 0) XtSetSensitive(root.home, True);
     if (root.back != 0) XtSetSensitive(root.back, True);
   }
-
   return;
 }
 
 /*
  * LoadDoc
- *
  */
 static Document *
-LoadDoc(up, reload, cacheonly)
+LoadDoc(up, reload, localonly)
 URLParts *up;
 int reload;
-int cacheonly;
+int localonly;			/* boolean;  in cache or on localhost */
 {
   Document *d;
   URLParts *tup = NULL, *nup = NULL;
+  static char *download = NULL;	/* as in DisplayTransferStatus --GN */
 
   if (root.dlist != NULL && reload == 0)
   {
@@ -678,27 +700,46 @@ int cacheonly;
   }
   else nup = up;
 
-  if (!root.cacheOff && !reload) d = ReadCache(nup);
-  else d = NULL;
-  if (d == NULL && !cacheonly)
+  d = (root.cacheOff  ||  reload) ?  NULL :  ReadCache (nup);
+
+  /*
+   * It might be nice to have a cheap, full-blown "Is this URL on the local
+   * host" test, but it's too expensive to strcmp the hostname of every image
+   * (when localonly == True) against the full set of gethostbyname() h_name
+   * and h_aliases[].  Checking for "localhost" at least handles the most
+   * common case.  Furthermore, there's not that much to be gained, since
+   * clicking on delayed local-but-not-"localhost" images now works.  The
+   * ability to test delayed and non-delayed image loading using just files
+   * on the local host might also be considered a feature.
+   */
+  if (d == NULL  &&
+       (!localonly  ||
+	nup->hostname != NULL  &&  strcmp (nup->hostname, "localhost") == 0))
   {
-    XDefineCursor(XtDisplay(root.toplevel), XtWindow(root.toplevel),
-                  root.watch);
-  
-    SetURL(nup);
-    SetTitle(GetFromStringDB("download"));
-    
+    if (!localonly)
+    {
+      XDefineCursor(XtDisplay(root.toplevel), XtWindow(root.toplevel),
+		    root.watch);
+
+      SetURL(nup);
+      if (download == NULL)  download = GetFromStringDB("download");
+      SetTitle(download);
+    }
+
     d = LoadDocument(nup, root.plist, root.mtlist, reload);
 
-    XDefineCursor(XtDisplay(root.toplevel), XtWindow(root.toplevel),
-		  root.left_ptr);
+    if (!localonly)
+    {
+      XDefineCursor(XtDisplay(root.toplevel), XtWindow(root.toplevel),
+		    root.left_ptr);
+    }
   }
+  if (d == NULL  ||  d->status != DS_ERROR)  /* take care --GN */
+    SetTitle("");
 
-  SetTitle("");
+  if (tup != NULL)  DestroyURLParts (tup);
 
-  if (tup != NULL) DestroyURLParts(tup);
-
-  return(d);
+  return (d);
 }
 
 /*
@@ -711,6 +752,8 @@ XtPointer cldata, cbdata;
 {
   DestroyDocument((Document *)cldata);
   XtDestroyWidget(w);
+  SetTitle(NULL);		/* restore title */
+  SetURL(root.dlist ? root.dlist->up : NULL);  /* restore URL */
 
   return;
 }
@@ -913,15 +956,14 @@ int download;
       XtPopup(nw, XtGrabNone);
     }
     else DestroyDocument(d);
-
     return;
   }
 
   /*
-   * Convert needs to be called in case there is a transfer or content
-   * encoding.
+   * Convert needs to be called in case there is a content transfer or content
+   * encoding.  But not for downloads, says Roman.
    */
-  if (d->ptext == NULL)
+  if (d->ptext == NULL && ! download)
   {
     /*
      * This frightens me.  I know it is bad but I can't think of anything
@@ -949,8 +991,27 @@ int download;
   if (d->status == DS_OK &&
       (download || (d->content == NULL && d->pcontent == NULL)))
   {
-    CreateStrReq("filename", root.savestr, False, SaveOCallback,
-		 SaveDCallback, (XtPointer)d);
+
+	/* Let's offer to save the downloaded file under the same name.
+        Roman Czyborra, 1995-04-13 */
+      
+      char *filename=NULL, *basename; int length;
+
+      if (d->up) if (filename= d->up->filename) if (length = strlen(filename))
+      {
+         if (filename[length-1]=='/')
+         {
+             filename[length-1]='\0';
+         }
+         if (basename = strrchr (filename, '/'))
+         {
+             filename = basename + 1;
+         }
+      }
+      if (! filename) filename="";
+
+      CreateStrReq("filename", filename, False, SaveOCallback,
+		   SaveDCallback, (XtPointer)d);
   }
   else if ((d->content != NULL && strcasecmp(d->content, "text/html") == 0) ||
 	   (d->pcontent != NULL && strcasecmp(d->pcontent, "text/html") == 0))
@@ -979,7 +1040,9 @@ int download;
       else
       {
 	DestroyDocument(d);
-	d = NULL;
+	SetTitle(NULL);		/* one of those cases where the Title
+				   used to be lost... --GN */
+	return;
       }
     }
     else
@@ -996,7 +1059,6 @@ int download;
   }
 
   DisplayCurrent();
-
   return;
 }
 
@@ -1012,6 +1074,7 @@ char *text;
 
   value = GetFromStringDB(text);
   d = BuildDocument(value, strlen(value), "text/html", 1, 0);
+  d->status = DS_ERROR;
   HandleDoc(d, 0);
 
   return;
@@ -1066,6 +1129,23 @@ WbAnchorCallbackData *c;
   return;
 }
 
+/* rwmcm */
+/*
+ * AnchorURLDisplay
+ *
+ * Called when the mouse moves over an anchor
+ */
+void
+AnchorURLDisplay(w, c)
+Widget w;
+char *c;
+{
+  Arg args[5];
+  
+  XtSetArg(args[0], XtNlabel, c);
+  XtSetValues(root.anchordisplay,args,1);
+}
+
 /*
  * OpenDocument
  *
@@ -1089,9 +1169,20 @@ char *url;
   else
   {
     if (root.loadstr) free_mem(root.loadstr);
-    root.loadstr = alloc_string(url);
+    /*
+     * short cut -- if user types in "foo.com" turn that into "http://foo.com/"
+     * this means you must type "./foo.com" if opening a relative URL by that
+     * name.  Colons and slashes disable this, so you can open mailto:foo@bar
+     * or news:alt.gourmand if you have mailto or news helper scripts.
+     * You can disable this by setting X resource openButtonShortcut False.
+     */
+    if (root.openButtonShortcut && !strchr(url, ':') && !strchr(url, '/')) {
+      root.loadstr = alloc_mem(strlen(url) + 9);
+      sprintf(root.loadstr, "http://%s/", url);
+    } else
+      root.loadstr = alloc_string(url);
 
-    up = ParseURL(url);
+    up = ParseURL(root.loadstr);
     if (up != NULL)
     {
       d = LoadDoc(up, 0, 0);
@@ -1698,7 +1789,7 @@ FileAction()
 }
 
 /*
- * sigiqh_handler
+ * sigigh_handler
  */
 static void
 sigigh_handler()
@@ -1711,13 +1802,21 @@ sigigh_handler()
 /*
  * Quit
  *
- * Called when the user clicks on the quit button
+ * Called when the user clicks on the quit button, or when handling a signal
+ * asking us to terminate cleanly.  Prevent re-entrant use of the signal
+ * handler, but allow SIGTERM to cause termination in case something is going
+ * seriously wrong during the cleanup phase.
  */
 void
 Quit(w, cldata, cbdata)
 Widget w;
 XtPointer cldata, cbdata;
 {
+  signal(SIGINT, SIG_IGN);
+  signal(SIGHUP, SIG_IGN);
+  signal(SIGALRM, SIG_IGN);
+  signal(SIGTERM, SIG_DFL);
+
   if (root.dlist && root.dlist->doc) DestroyDocument(root.dlist->doc);
 
   if (!root.cacheOff) CleanCache();
@@ -1757,6 +1856,10 @@ XtPointer cldata, cbdata;
     if ((data = BuildDocumentInfo(root.dlist->doc)) != NULL)
     {
       d = BuildDocument(data, strlen(data), "text/plain", 0, 0);
+      if (root.dlist->doc->up != (URLParts *)NULL)
+      {
+	d->up = DupURLParts(root.dlist->doc->up);
+      }
       HandleDoc(d, 0);
     }
   }
@@ -1804,7 +1907,7 @@ ImageInfo *
 ImageResolve(w, url, delay)
 Widget w;
 char *url;
-int delay;
+int delay;	/* boolean; if true, only if in cache or on localhost */
 {
   Document *d, *x;
   URLParts *tup;
@@ -1825,8 +1928,7 @@ int delay;
   tup = ParseURL(url);
   if (tup == NULL) return(NULL);
 
-  if (delay) d = LoadDoc(tup, 0, 1);
-  else d = LoadDoc(tup, 0, 0);
+  d = LoadDoc (tup, 0, delay);
 
   DestroyURLParts(tup);
 
@@ -1882,19 +1984,74 @@ int delay;
   return(i);
 }
 
+
 /*
- * DelayedImageResolve
- *
- * Called when the user clicks on a delayed image.
+ * DeferPix
  */
-ImageInfo *
-DelayedImageResolve(w, url)
+void
+DeferPix(w, cldata, cbdata)
 Widget w;
-char *url;
+XtPointer cldata, cbdata;
 {
-  return(ImageResolve(w, url, 0));
+  extern void SetDelayedImageLoading();	 /* in libhtmlw/HTML.c */
+
+  SetDelayedImageLoading (root.w, cbdata ? True : False);
+
+  return;
 }
 
+/*
+ * DeferPixAction
+ */
+static void
+DeferPixAction()
+{
+  Boolean state;
+  Widget w = root.deferpix;
+
+  if (w != NULL)
+  {
+    XtVaGetValues (w, XtNstate, &state, NULL);
+    state =  (state == False) ? True : False;
+    XtVaSetValues (w, XtNstate, state, NULL);
+
+    DeferPix(w, &root, state);
+  }
+  else				/* user chose not to have this button */
+  {
+    extern void ToggleDelayedImageLoading();  /* in libhtmlw/HTML.c */
+
+    /*
+     * Without the DeferPix button, there's no way to indicate explicitly
+     * which mode is current, but the user can tell by what happens.
+     * Toggle the delay_images state.
+     */
+    ToggleDelayedImageLoading (root.w);
+  }
+
+  return;
+}
+
+/*
+ * SetDeferPixButtonState
+ *
+ * Allows initialization of the button state from the
+ * X resource *delayImageLoads.
+ */
+void
+SetDeferPixButtonState (newstate)
+int newstate;
+{
+  Widget w = root.deferpix;
+
+  if (w != NULL)
+  {
+    newstate = newstate ?  True : False;  /* canonicalize state */
+    XtVaSetValues (w, XtNstate, newstate, NULL);
+  }
+}
+
+
 /*
  * BookmarkCallback
  */
@@ -1969,7 +2126,7 @@ BookmarkAction()
   unsigned int mask;
   char *filename;
 
-  filename = FixFilename(root.bookmarkFile);
+  filename = alloc_string(FixFilename(root.bookmarkFile));
 
   XQueryPointer(XtDisplay(root.toplevel), XtWindow(root.toplevel),
 		&rw, &cw,
@@ -2052,31 +2209,150 @@ char *url;
   return(0);
 }
 
+/*
+ * DisplayConnectStatus
+ *
+ * This is called by the networking and data transfer functions to display
+ * the status of a DNS lookup, a socket being connected, or an outbound
+ * request, and to poll the Cancel button.
+ *
+ * First arg tells us which message to show, second is a hostname or IP
+ * address  (as a dotted quad string)  or NULL.
+ * --GN 1997Apr30
+ */
+int
+DisplayConnectStatus(msg, host)
+int msg;
+char *host;
+{
+  static char *DNSquery = NULL;	/* msgs 0...3 use the host arg */
+  static char *TCPconnecting = NULL;
+  static char *TCPconnected = NULL;
+  static char *TCPsendreq = NULL;
+  static char *TCPsentawait = NULL; /* msg 4 ignores it */
+  static char *download = NULL;	/* should never happen */
+  static char ourhost[204];
+
+  if (host != NULL)
+  {
+    if (strlen(host) < 204)
+      strcpy(ourhost, host);
+    else
+    {
+      strncpy(ourhost, host, 200);
+      strcpy(ourhost+200, "...");
+    }
+  } /* else re-use previous hostname, which we got from net_open */
+
+  if (root.titledisplay != 0)
+  {
+    char buffer[256];
+
+    switch (msg)
+    {
+    case 0:
+      if (DNSquery == NULL) DNSquery = GetFromStringDB("DNSquery");
+      sprintf(buffer, DNSquery, ourhost);
+      break;
+    case 1:
+      if (TCPconnecting == NULL)
+	TCPconnecting = GetFromStringDB("TCPconnecting");
+      sprintf(buffer, TCPconnecting, ourhost);
+      break;
+    case 2:
+      if (TCPconnected == NULL)
+	TCPconnected = GetFromStringDB("TCPconnected");
+      sprintf(buffer, TCPconnected, ourhost);
+      break;
+    case 3:
+      if (TCPsendreq == NULL) TCPsendreq = GetFromStringDB("TCPsendreq");
+      sprintf(buffer, TCPsendreq, ourhost);
+      break;
+    case 4:
+      if (TCPsentawait == NULL) TCPsentawait = GetFromStringDB("TCPsentawait");
+      strcpy(buffer, TCPsentawait);
+      break;
+    default:
+      if (download == NULL) download = GetFromStringDB("download");
+      strcpy(buffer, download);
+      break;
+    }
+
+    SetTitle(buffer);
+  }
+
+  if (root.cancel)
+  {
+    XEvent xe;
+
+    if (XCheckWindowEvent(XtDisplay(root.cancel), XtWindow(root.cancel),
+			  ButtonReleaseMask, &xe))
+    {
+      root.cancelop = True;
+      return(1);
+    }
+  }
+
+  return(0);
+}
+
 /*
  * DisplayTransferStatus
  *
  * This is called by the data transfer functions to display the status of
- * a transfer.
+ * an inbound transfer, and to poll the Cancel button.
+ *
+ * When max < 0, we're reading HTTP response headers or Gopher menus or
+ * FTP control channel chats, so we say "Reading response: mumble bytes".
+ * When max == 0, we're reading data but don't know how much to expect,
+ * so we say "Downloading document..." and then "mumble bytes loaded".
+ * When max > 0, we expect max bytes, and say "mumble bytes of out max loaded".
+ * --GN 1997Apr30
  */
 int
-DisplayTransferStatus(n, max)
-int n, max;
+DisplayTransferStatus(n, max, showit)
+int n, max, showit;
 {
   static int count = 0;
   static char *byte2 = NULL;
   static char *byte1 = NULL;
+  static char *byte0 = NULL;
+  static char *byte21 = NULL;
+  static char *byte11 = NULL;
+  static char *byte01 = NULL;
+
+  if (showit) count = 0;	/* force an up-to-date display */
 
   if (root.titledisplay != 0 &&
-      n > 0 &&
-      count++ % root.statusUpdate == 0)
+      count++ % (root.statusUpdate * (n > 0 && max < 0 ? 5 : 1)) == 0)
   {
-    char buffer[256];
-
-    if (byte2 == NULL) byte2 = GetFromStringDB("byte2");
-    if (byte1 == NULL) byte1 = GetFromStringDB("byte1");
-
-    if (max > 0) sprintf (buffer, byte2, n, max);
-    else sprintf (buffer, byte1, n);
+    char buffer[256];		/* Sheesh.  Probably safe though unless s.o.
+				   introduces 240-digit integers. --GN */
+    if (max < 0)
+    {
+      if (n == 0 && max == -8)        /* still waiting for start of response */
+      {
+      return(DisplayConnectStatus(4, NULL));
+      }
+      if (byte0 == NULL) byte0 = GetFromStringDB("byte0");
+      if (byte01 == NULL) byte01 = GetFromStringDB("byte01");
+      if (n == 1) strcpy (buffer, byte01);
+      else sprintf (buffer, byte0, n);
+    }
+    else if (max > 0)
+    {
+      if (byte2 == NULL) byte2 = GetFromStringDB("byte2");
+      if (byte21 == NULL) byte21 = GetFromStringDB("byte21");
+      if (n == 1) sprintf (buffer, byte21, max);
+      else sprintf (buffer, byte2, n, max);
+    }
+    else
+    {
+      if (byte1 == NULL) byte1 = GetFromStringDB("byte1");
+      if (byte11 == NULL) byte1 = GetFromStringDB("byte11");
+      if (n == 1) strcpy (buffer, byte11);
+      else sprintf (buffer, byte1, n);
+    }
 
     SetTitle(buffer);
   }
